package com.xtremelabs.robolectric.shadows;


import android.graphics.ImageFormat;
import android.hardware.Camera;
import com.xtremelabs.robolectric.Robolectric;
import com.xtremelabs.robolectric.WithTestDefaultsRunner;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.List;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.not;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.hamcrest.core.IsCollectionContaining.hasItem;
import static org.junit.Assert.assertThat;

@RunWith(WithTestDefaultsRunner.class)
public class CameraParametersTest {

    private Camera.Parameters parameters;
    private ShadowCameraParameters shadowParameters;

    @Before
    public void setUp() throws Exception {
        parameters = Robolectric.newInstanceOf(Camera.Parameters.class);
        shadowParameters = Robolectric.shadowOf(parameters);
    }

    @Test
    public void testSetPreviewSize() throws Exception {
        assertThat(shadowParameters.getPreviewWidth(), not(equalTo(640)));
        assertThat(shadowParameters.getPreviewHeight(), not(equalTo(480)));
        parameters.setPreviewSize(640, 480);
        assertThat(shadowParameters.getPreviewWidth(), equalTo(640));
        assertThat(shadowParameters.getPreviewHeight(), equalTo(480));
    }

    @Test
    public void testPreviewFormat() throws Exception {
        assertThat(shadowParameters.getPreviewFormat(), equalTo(ImageFormat.NV21));
        parameters.setPreviewFormat(ImageFormat.JPEG);
        assertThat(shadowParameters.getPreviewFormat(), equalTo(ImageFormat.JPEG));
    }

    @Test
    public void testGetSupportedPreviewFormats() throws Exception {
        List<Integer> supportedFormats = parameters.getSupportedPreviewFormats();
        assertThat(supportedFormats, notNullValue());
        assertThat(supportedFormats.size(), not(equalTo(0)));
        assertThat(supportedFormats, hasItem(ImageFormat.NV21));
    }
}
